/*-----------------------------------------------------------------------

File  : parser.c

Author: Stephan Schulz

Contents
 
  Simple PL parser

  Copyright 1998-2002 by the author.
  This code is released under the GNU General Public Licence.
  See the file COPYING in the main CLIB directory for details.
  Run "eprover -h" for contact information.

Changes

<1> Wed Oct  2 00:23:44 CEST 2002
    New

-----------------------------------------------------------------------*/

#include "parser.h"



/*---------------------------------------------------------------------*/
/*                        Global Variables                             */
/*---------------------------------------------------------------------*/

static long statement_count = 0;
Token current;

/*---------------------------------------------------------------------*/
/*                      Forward Declarations                           */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                         Internal Functions                          */
/*---------------------------------------------------------------------*/


/*-----------------------------------------------------------------------
//
// Function: accept_token()
//
//   If the current token is of type type, read the next token, else
//   print an error message and terminate.
//
// Global Variables: -
//
// Side Effects    : -
//
/----------------------------------------------------------------------*/

void accept_token(TokenType type)
{
   if(current.type == type)
   {
      current = NextToken();
   }
   else
   {
      ScannerError(current.line, current.column, "Unexpected token");
   }
}

/*---------------------------------------------------------------------*/
/*                         Exported Functions                          */
/*---------------------------------------------------------------------*/


/*-----------------------------------------------------------------------
//
// Function: PrintStatement()
// 
//   Print a PL statement in internal encoding back as PL  
//
// Global Variables: -
//
// Side Effects    : Output
//
/----------------------------------------------------------------------*/

void PrintStatement(Statement_p statement)
{
   if(statement->label)
   {
      printf("%s: ", statement->label);      
   }
   switch(statement->type)
   {
   case StInc:
	 printf("%s++;\n",statement->argument);
	 break;
   case StNull:
	 printf("%s<=0;\n",statement->argument);
	 break;
   case StGoto:
	 printf("goto %s;\n",statement->argument);
	 break;
   case StLoop:
	 printf("loop %s;\n",statement->argument);
	 PrintProgram(statement->subprog);
	 printf("endloop;\n");
	 break;
   default:
	 Error("Unknown statement type in statement");
	 break;
   }
}


/*-----------------------------------------------------------------------
//
// Function: PrintProgram()
//
//   Print a PL program back.
//
// Global Variables: -
//
// Side Effects    : Output
//
/----------------------------------------------------------------------*/

void PrintProgram(Prog_p prog)
{
   if(prog)
   {
      PrintStatement(prog->statement);
      PrintProgram(prog->prog_rest);
   }
}


/*-----------------------------------------------------------------------
//
// Function: ParseStatement()
//
//   Parse a (possibly labeled) statement.
//
// Global Variables: current
//
// Side Effects    : Reads input, allocates memory
//
/----------------------------------------------------------------------*/

Statement_p ParseStatement()
{
   Statement_p res = SecureMalloc(sizeof *res);
   
   res->statement_id = statement_count++;
   res->label = NULL;
   res->argument = NULL;
   res->subprog = NULL;
   if(current.type == Label)
   {
      res->label = SecureStrdup(current.spelling);
      accept_token(Label);
      accept_token(Colon);
   }
   switch(current.type)
   {
   case Variable:
	 res->argument = SecureStrdup(current.spelling);
	 accept_token(Variable);
	 if(current.type == AssignOp)
	 {
	    res->type = StNull;
	    accept_token(AssignOp);
	    accept_token(NullDigit);
	 }
	 else
	 {
	    res->type = StInc;
	    accept_token(IncOp);
	 }
	 accept_token(Semicolon);
	 break;
   case ResWordLoop:
	 res->type = StLoop;
	 accept_token(ResWordLoop);
	 res->argument = SecureStrdup(current.spelling);
	 accept_token(Variable);
	 accept_token(Semicolon);
	 res->subprog = ParseProg();
	 accept_token(ResWordEndloop);
	 accept_token(Semicolon);
	 break;
   case ResWordGoto:
	 res->type = StGoto;
	 accept_token(ResWordGoto);
	 res->argument = SecureStrdup(current.spelling);
	 accept_token(Label);
	 accept_token(Semicolon);
	 break;
   default:	 
	 ScannerError(current.line, current.column, "Unexpected token");
	 break;
   }
   return res;
}


/*-----------------------------------------------------------------------
//
// Function:  ParseProg()
//
//   Parse a PL program into the internal structure.
//
// Global Variables: current
//
// Side Effects    : Read Input, allocates memory
//
/----------------------------------------------------------------------*/

Prog_p ParseProg()
{
   Prog_p res = NULL;
   
   if(current.type != NoToken && current.type!=ResWordEndloop)
   {
      res = SecureMalloc(sizeof *res);
      res->statement = ParseStatement();
      res->prog_rest = ParseProg();
   }
   return res;
}

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/




