/*-----------------------------------------------------------------------

File  : utilities.c

Author: Stephan Schulz

Contents
 
  Simple utility functions.

  Copyright 2002 by the author.
  This code is released under the GNU General Public Licence.
  See the file COPYING.

Changes

<1> Wed Sep 11 01:18:12 CEST 2002
    New

-----------------------------------------------------------------------*/

#include "utilities.h"



/*---------------------------------------------------------------------*/
/*                        Global Variables                             */
/*---------------------------------------------------------------------*/

char* ProgName = "unknown";

/*---------------------------------------------------------------------*/
/*                      Forward Declarations                           */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                         Internal Functions                          */
/*---------------------------------------------------------------------*/



/*---------------------------------------------------------------------*/
/*                         Exported Functions                          */
/*---------------------------------------------------------------------*/


/*-----------------------------------------------------------------------
//
// Function: Error()
//
//   Print an error message and fail.
//
// Global Variables: 
//
// Side Effects    : 
//
/----------------------------------------------------------------------*/

void Error(char* message)
{
   fprintf(stderr, "%s: %s\n", ProgName, message);
   exit(EXIT_FAILURE);
}

/*-----------------------------------------------------------------------
//
// Function: SecureMalloc()
//
//   As malloc, but terminate with an error if out of memory.
//
// Global Variables: -
//
// Side Effects    : -
//
/----------------------------------------------------------------------*/

void* SecureMalloc(int size)
{
   void* res = malloc(size);

   if(!res)
   {
      Error("malloc() failure -- out of memory?");
   }
   return res;
}


/*-----------------------------------------------------------------------
//
// Function: SecureRealloc()
//
//   As realloc(), but terminate with an error if out of memory.
//
// Global Variables: 
//
// Side Effects    : 
//
/----------------------------------------------------------------------*/

void* SecureRealloc(void *ptr, int size)
{
   void* res = realloc(ptr, size);

   if(!res)
   {
      Error("realloc() failure -- out of memory?");
   }
   return res;   
}


/*-----------------------------------------------------------------------
//
// Function: SecureStrdup()
//
//   As strdup(), but terminate with an error if out of memory.
//
// Global Variables: 
//
// Side Effects    : 
//
/----------------------------------------------------------------------*/

char* SecureStrdup(char* source)
{
   void* res = SecureMalloc(strlen(source)+1);

   if(!res)
   {
      Error("SecureStrdup() failure -- out of memory?");
   }
   strcpy(res, source);
   return res;  
}


/*-----------------------------------------------------------------------
//
// Function: DynastringInit()
//
//   Initialize the dynamic string structure.
//
// Global Variables: -
//
// Side Effects    : -
//
/----------------------------------------------------------------------*/

void  DynastringInit(dynastring_p str)
{
   str->str = SecureMalloc(DYNASTR_GROW);
   str->len = 0;
   str->mem = 0;
}


/*-----------------------------------------------------------------------
//
// Function: DynastringReset()
//
//   Reset the string to empty.
//
// Global Variables: -
//
// Side Effects    : -
//
/----------------------------------------------------------------------*/

void  DynastringReset(dynastring_p str)
{
   assert(str->str);
   str->len = 0;
   str->str[0] = '\0';
}


/*-----------------------------------------------------------------------
//
// Function: DynastringFreeMem()
//
//   Free the memory taken by a dynamic string.
//
// Global Variables: -
//
// Side Effects    : -
//
/----------------------------------------------------------------------*/

void  DynastringFreeMem(dynastring_p str)
{
   assert(str->str);
   free(str->str);
   str->str = NULL;
   str->mem = 0;
}

/*-----------------------------------------------------------------------
//
// Function: DynastringAppendChar()
//
//   Append a character to a dynastring.
//
// Global Variables: -
//
// Side Effects    : -
//
/----------------------------------------------------------------------*/

char* DynastringAppendChar(dynastring_p str, char c)
{
   if(str->len == str->mem-1)
   {
      str->mem +=DYNASTR_GROW;
      str->str = SecureRealloc(str, str->mem);
   }
   str->str[str->len] = c;
   str->len++;
   str->str[str->len] = '\0';
   return str->str;
}

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/














